// SPDX-License-Identifier: GPL-2.0-or-later
/*
 * Copyright (C) 2021 Verisilicon Inc.
 *
 * This is hdma transmission driver for Linux.
 */
#include "gc_hal.h"
#include "gc_hal_kernel.h"
#include "gc_hal_kernel_debug.h"
#include "gc_hal_osal_support.h"

//static int osal_pid_global[OSAL_KTHREAD_MAX_NUM];
static struct osal_pid_mapping osal_pid_map[OSAL_KTHREAD_MAX_NUM * OSAL_KTHREAD_MAX_NUM];
static gcmkDECLARE_MUTEX(g_osal_pid_mutex);

gceSTATUS gc_hal_osal_add_mapping(int kpid, int pid)
{
    int i = 0;

    gcmkMUTEX_LOCK(g_osal_pid_mutex);
    for (i = 0; i < OSAL_KTHREAD_MAX_NUM * OSAL_KTHREAD_MAX_NUM; i++) {
        if (!osal_pid_map[i].osal_flag) {
            osal_pid_map[i].osal_flag = 1;
            break;
        }
    }

    if (i < OSAL_KTHREAD_MAX_NUM * OSAL_KTHREAD_MAX_NUM) {
        osal_pid_map[i].osal_kpid = kpid;
        osal_pid_map[i].osal_pid = pid;
        gcmkMUTEX_UNLOCK(g_osal_pid_mutex);
        return 0;
    }

    pr_err("out of map total size %d.\n", i);

    gcmkMUTEX_UNLOCK(g_osal_pid_mutex);
    return gcvSTATUS_OUT_OF_RESOURCES;
}

void gc_hal_osal_remove_single_mapping(int kpid) {
    int i = 0;

    gcmkMUTEX_LOCK(g_osal_pid_mutex);
    for (i = 0; i < OSAL_KTHREAD_MAX_NUM * OSAL_KTHREAD_MAX_NUM; i++) {
        if (osal_pid_map[i].osal_flag && osal_pid_map[i].osal_kpid == kpid) {
            osal_pid_map[i].osal_flag = 0;
            osal_pid_map[i].osal_kpid = 0;
            osal_pid_map[i].osal_pid = 0;
            break;
        }
    }
    gcmkMUTEX_UNLOCK(g_osal_pid_mutex);
}

void gc_hal_osal_remove_mapping(int pid)
{
    int i = 0;

    for (i = 0; i < OSAL_KTHREAD_MAX_NUM * OSAL_KTHREAD_MAX_NUM; i++) {
        if (osal_pid_map[i].osal_flag && osal_pid_map[i].osal_pid == pid) {
            osal_pid_map[i].osal_flag = 0;
            osal_pid_map[i].osal_kpid = 0;
            osal_pid_map[i].osal_pid = 0;
        }
    }
}

int gc_hal_osal_get_pid(int kpid)
{
    int i = 0;

    for (i = 0; i < OSAL_KTHREAD_MAX_NUM * OSAL_KTHREAD_MAX_NUM; i++) {
        if (osal_pid_map[i].osal_flag && osal_pid_map[i].osal_kpid == kpid)
            return osal_pid_map[i].osal_pid;
    }
    return -1;
}
